using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using System.Text;

namespace FlvComposer
{
    //=====================================================================================//
    //                         internal sealed class NativeMethods                         //
    //=====================================================================================//
    internal sealed class NativeMethods
    {
        internal delegate void FTWRITEDATACALLBACK(IntPtr lpParam, IntPtr lpData, Int32 nLen);

        //=====================================================================================//
        //                               Video Codec Enumeration                               //
        //=====================================================================================//
        internal const int FT_VIDEOCODEC_SCREENVIDEO    = 0x03;

        //=====================================================================================//
        //                               Audio Codec Enumeration                               //
        //=====================================================================================//
        internal const int FT_SOUNDFORMAT_UNCOMPRESSED  = 0x00;

        internal const int FT_SOUNDRATE_5KHZ    = 0x00;
        internal const int FT_SOUNDRATE_11KHZ   = 0x01;
        internal const int FT_SOUNDRATE_22KHZ   = 0x02;
        internal const int FT_SOUNDRATE_44KHZ   = 0x03;

        internal const int FT_SOUNDSIZE_8BIT    = 0x00;
        internal const int FT_SOUNDSIZE_16BIT   = 0x01;

        internal const int FT_SOUNDTYPE_MONO    = 0x00;
        internal const int FT_SOUNDTYPE_STEREO  = 0x01;
        
        //=====================================================================================//
        //                                FTVIDEOINFO structure                                //
        //=====================================================================================//
        [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi)]
        internal struct FTVIDEOINFO
        {
	        internal Int32 dwCodec;
	        internal Int32 dwWidth;
	        internal Int32 dwHeight;
        }

        //=====================================================================================//
        //                                FTAUDIOINFO structure                                //
        //=====================================================================================//
        [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi)]
        internal struct FTAUDIOINFO
        {
            internal Int32 dwFormat;
            internal Int32 dwRate;
            internal Int32 dwSize;
            internal Int32 dwType;
        }

        //=====================================================================================//
        //                              FTFLVSTREAMINFO structure                              //
        //=====================================================================================//
        [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi)]
        internal struct FTFLVSTREAMINFO
        {
            internal IntPtr lpVideoInfo;
            internal IntPtr lpAudioInfo;
            [MarshalAs(UnmanagedType.FunctionPtr)]
            internal FTWRITEDATACALLBACK lpWriteDataCallback;
            internal IntPtr lpParam;
        }

        //=====================================================================================//
        //                                FLV Stream interface                                 //
        //=====================================================================================//
        ///	Create FLV stream
        [DllImport("flashtool.dll")]
        internal static extern IntPtr ftFlvStreamCreate(ref FTFLVSTREAMINFO lpFlvStreamInfo);

        ///	Destroy FLV stream
        [DllImport("flashtool.dll")]
        internal static extern void ftFlvStreamDestroy(IntPtr hFlvStream);

        /// Add video tag to FLV stream
        [DllImport("flashtool.dll")]
        internal static extern void ftFlvStreamPutVideoData(IntPtr hFlvStream, IntPtr lpData, Int32 nLen, Int32 nTimeStamp);

        ///	Add audio tag to FLV stream
        [DllImport("flashtool.dll")]
        internal static extern void ftFlvStreamPutAudioData(IntPtr hFlvStream, IntPtr lpData, Int32 nLen, Int32 nTimeStamp);

        ///	Set context secure key
        [DllImport("flashtool.dll")]
        internal static extern void ftSetContextKey([MarshalAs(UnmanagedType.LPStr)] string szKey);

        //=====================================================================================//
        //                            ScreenVideo encoder interface                            //
        //=====================================================================================//
        ///	Create ScreenVideo encoder
        [DllImport("flashtool.dll")]
        internal static extern IntPtr ftScreenVideoEncoderCreate(Int32 dwWidth, Int32 dwHeight, Int32 dwBlockSize);

        ///	Destroy ScreenVideo encoder
        [DllImport("flashtool.dll")]
        internal static extern void ftScreenVideoEncoderDestroy(IntPtr hScreenVideoEncoder);

        ///	Encode row video frame to ScreenVideo frame
        [DllImport("flashtool.dll")]
        internal static extern bool ftScreenVideoEncodeFrame(IntPtr hScreenVideoEncoder, IntPtr lpData, Int32 cbBytes, bool bFlipY);

        ///	Get current encoded frame data
        [DllImport("flashtool.dll")]
        internal static extern void ftScreenVideoGetEncodedFrame(IntPtr hScreenVideoEncoder, ref IntPtr lppData, ref Int32 lpcbBytes);

    }
}
