unit FlvCompiler;

interface

uses
    Windows, Classes, Graphics, SysUtils,
    NativeMethods;

type
    TFlvCompiler = class(TObject)
    private
      m_width: Integer;
      m_height: Integer;
      m_hFlvStream: Integer;
      m_fileStream: TFileStream;
      m_hScreenVideoEncoder: Integer;
      function CheckFlvStream(width: Integer; height: Integer): Boolean;
    public
      constructor Create(fileName: String);
      destructor Destroy; override;
      function AddFrame(imageFileName: String; timestamp: Integer): Boolean;
    end;

    procedure WriteDataCallback(lpParam: Pointer; const lpData: Pointer; nLen: Integer); stdcall;

implementation

constructor TFlvCompiler.Create(fileName: String);
begin
     m_width := 0;
     m_height := 0;
     m_hFlvStream := 0;
     m_hScreenVideoEncoder := 0;
     m_fileStream := TFileStream.Create(fileName,fmCreate);
end;

destructor TFlvCompiler.Destroy;
begin
     // release Screen Video encoder
     if m_hScreenVideoEncoder <> 0 then
     begin
          ftScreenVideoEncoderDestroy(m_hScreenVideoEncoder);
     end;
     // release Flv Stream
     if m_hFlvStream <> 0 then
     begin
          ftFlvStreamDestroy(m_hFlvStream);
     end;
     // close output file
     m_fileStream.Free();

     inherited Destroy;
end;

function TFlvCompiler.AddFrame(imageFileName: String; timestamp: Integer): Boolean;
var
   bitmap: TBitmap;
   stride: Integer;
   size: Integer;
   data: PChar;
   i: Integer;
begin
     try
        bitmap := TBitmap.Create();
        bitmap.LoadFromFile(imageFileName);
        if bitmap.PixelFormat = pf24bit then
        begin
             if CheckFlvStream(bitmap.Width, bitmap.Height) then
             begin
                  if m_hScreenVideoEncoder <> 0 then
                  begin
                       stride := (bitmap.Width * (24 div 8) + 3) and not 3;
                       size := stride * bitmap.Height;
                       GetMem(data,size);
                       ZeroMemory(data,size);

                       // copy bitmap image to temporary buffer
                       for i := 0 to bitmap.Height-1 do
                       begin
                            CopyMemory(data+(i*stride),bitmap.ScanLine[i],stride);
                       end;

                       // encode the video frame
                       ftScreenVideoEncodeFrame(m_hScreenVideoEncoder,data,size,false);
                       FreeMem(data);

                       // get the encoded video frame
                       ftScreenVideoGetEncodedFrame(m_hScreenVideoEncoder,@data,@size);

                       // add frame to Flv stream
                       ftFlvStreamPutVideoData(m_hFlvStream,data,size,timestamp);

                       // release bitmap and exit
                       Result := true;
                       bitmap.Free();
                       exit;
                  end;
             end;
        end;
     except
        on e: Exception do
        begin
             bitmap.Free();
             MessageBox(0, PChar(e.Message), PChar('Flash Video Composer'), MB_OK + MB_ICONERROR);
        end;
     end;

     Result := false;
end;

function TFlvCompiler.CheckFlvStream(width: Integer; height: Integer): Boolean;
var
   ftflvsi: FTFLVSTREAMINFO;
   ftvi: FTVIDEOINFO;
begin
     if m_hFlvStream = 0 then
     begin
          // initialize FTVIDEOINFO struct
          ftvi.dwCodec := FT_VIDEOCODEC_SCREENVIDEO;
          ftvi.dwWidth := width;
          ftvi.dwHeight := height;

          // initialize FTFLVSTREAMINFO struct
          ftflvsi.lpVideoInfo := @ftvi;
          ftflvsi.lpAudioInfo := nil;
          ftflvsi.lpWriteDataCallback := WriteDataCallback;
          ftflvsi.lpParam := @m_fileStream;

          // create flv stream
          m_hFlvStream := ftFlvStreamCreate(@ftflvsi);

          // create Screen Video encoder
          m_hScreenVideoEncoder := ftScreenVideoEncoderCreate(width,height,16);

          m_width := width;
          m_height := height;
     end;

     Result := (m_hFlvStream <> 0) and (m_width = width) and (m_height = height);
end;

procedure WriteDataCallback(lpParam: Pointer; const lpData: Pointer; nLen: Integer);
var
   fileStream: ^TFileStream;
begin
     fileStream := lpParam;
     fileStream.Write(lpData^,nLen);
end;

end.
